package com.jobs.service;

import io.minio.GetObjectArgs;
import io.minio.MinioClient;
import io.minio.PutObjectArgs;
import io.minio.RemoveObjectArgs;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.io.IOUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.web.multipart.MultipartFile;

import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.InputStream;

@Slf4j
@Service
public class MinioService {

    @Autowired
    private MinioClient minioClient;

    @Value("${minio.endpoint}")
    private String endPoint;

    @Value("${minio.bucket}")
    private String bucket;

    /**
     * 获取文件在 minio server 上的完整访问路径 url 地址
     *
     * @param minioPath 相对于 bucket 根目录的文件全路径
     */
    public String getFileFullUrl(String minioPath) {
        StringBuilder sb = new StringBuilder();
        sb.append(endPoint);
        sb.append(endPoint.endsWith("/") ? "" : "/");
        sb.append(bucket).append("/");
        sb.append(minioPath);
        return sb.toString();
    }

    /**
     * @param path 相对于 bucket 根目录的目录路径
     *             比如 /book/chinese 表示上传到 book 目录下的 chinese 目录中
     * @param file 上传的文件
     * @return 相对于 bucket 目录的文件全路径
     */
    public String uploadFile(String path, MultipartFile file) {
        try {
            return uploadFile(path, file.getOriginalFilename(), file.getInputStream(), file.getContentType());
        } catch (Exception ex) {
            log.error("minio 上传文件失败：", ex);
            return null;
        }
    }

    //上传文件
    public String uploadFile(String path, File file, String contentType) {
        try {
            FileInputStream inputStream = new FileInputStream(file);
            return uploadFile(path, file.getName(), inputStream, contentType);
        } catch (Exception ex) {
            log.error("minio 上传文件失败：", ex.getMessage());
            return null;
        }
    }

    //上传文件
    private String uploadFile(String path, String fileName,
                              InputStream inputStream, String contentType) throws Exception {
        //拼接上传文件的目录存储路径，如果目录不存在，则自动创建（支持多级目录的自动创建）
        String filePath = fileName;
        if (StringUtils.isNotBlank(path)) {
            StringBuilder sb = new StringBuilder();
            sb.append(path);
            sb.append(path.endsWith("/") ? "" : "/");
            sb.append(fileName);
            filePath = sb.toString();
        }

        PutObjectArgs putObjectArgs = PutObjectArgs.builder()
                .object(filePath).contentType(contentType).bucket(bucket)
                //inputStream.available() 表示文件的总字节数大小，-1 表示上传所有字节数
                .stream(inputStream, inputStream.available(), -1).build();
        minioClient.putObject(putObjectArgs);
        return filePath;
    }

    /**
     * 下载文件
     *
     * @param minioPath 相对于 bucket 根目录的文件全路径
     */
    public byte[] downloadFile(String minioPath) {
        if (StringUtils.isBlank(minioPath)) {
            return null;
        }

        try {
            GetObjectArgs getObjectArgs = GetObjectArgs.builder()
                    .bucket(bucket).object(minioPath).build();
            InputStream inputStream = minioClient.getObject(getObjectArgs);
            ByteArrayOutputStream outputStream = new ByteArrayOutputStream();
            IOUtils.copy(inputStream, outputStream);
            return outputStream.toByteArray();
        } catch (Exception ex) {
            log.error("minio 下载文件失败，pathUrl:{}/{}，失败信息：{}",
                    bucket, minioPath, ex.getMessage());
            return null;
        }
    }

    /**
     * 删除文件
     *
     * @param minioPath 相对于 bucket 根目录的文件全路径
     */
    public boolean deleteFile(String minioPath) {
        if (StringUtils.isBlank(minioPath)) {
            return true;
        }

        try {
            RemoveObjectArgs removeObjectArgs = RemoveObjectArgs.builder()
                    .bucket(bucket).object(minioPath).build();
            minioClient.removeObject(removeObjectArgs);
            return true;
        } catch (Exception ex) {
            log.error("minio 删除文件失败，pathUrl:{}/{}，失败信息：{}",
                    bucket, minioPath, ex.getMessage());
            return false;
        }
    }
}
